package org.bitbucket.bradleysmithllc.module_signer_mojo;

import com.sun.codemodel.*;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringEscapeUtils;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.apache.maven.project.MavenProjectHelper;
import org.eclipse.jgit.api.Git;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.eclipse.jgit.lib.Repository;
import org.eclipse.jgit.lib.RepositoryBuilder;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * Goal which touches a timestamp file.
 *
 * @goal package-meta-info
 * @phase process-sources
 */
public class ProjectSignerMOJO
		extends AbstractMojo
{
	/**
	 * @parameter expression="${outputDirectory}" default-value="${project.build.directory}/generated-sources/module-signer-source/"
	 * @required
	 */
	private File srcDirectory;

	/**
	 * @parameter expression="${outputDirectory}" default-value="${project.build.directory}/generated-sources/module-signer-resource/META-INF/services"
	 * @required
	 */
	private File rsrcDirectory;

	/**
	 * @parameter expression="${project}" default-value="${project}"
	 */
	protected MavenProject mavenProject;

	/**
	 * Helper class to assist in attaching artifacts to the project instance.
	 * project-helper instance, used to make addition of resources simpler.
	 * @component
	 * @required
	 * @readonly
	 */
	private MavenProjectHelper projectHelper;

	public void execute()
			throws MojoExecutionException
	{
		String package_ = clean(mavenProject.getGroupId()) + "." + clean(mavenProject.getArtifactId()) + ".version_" + clean(mavenProject.getVersion());

		try
		{
			String directory = "target/generated-sources/module-signer-resource";
			List includes = Collections.singletonList("**/*");
			List excludes = null;

			projectHelper.addResource(mavenProject, directory, includes, excludes);

			JCodeModel cm = new JCodeModel();
			JDefinedClass dc = cm._class(package_ + "." + "MavenProjectInstance");
			dc._extends(MavenProjectImpl.class);

			JMethod m = dc.constructor(0);
			m.mods().setPublic();

			JBlock block = m.body();

			block.directStatement("// System Properties");
			for (Map.Entry<Object, Object> propertyEntry : System.getProperties().entrySet())
			{
				block.directStatement("put(\"" + propertyEntry.getKey() + "\", \"" + StringEscapeUtils.escapeJava(String.valueOf(propertyEntry.getValue())) + "\");");
			}

			block.directStatement("// Project Properties");
			for (Map.Entry<Object, Object> propertyEntry : mavenProject.getProperties().entrySet())
			{
				block.directStatement("put(\"" + propertyEntry.getKey() + "\", \"" + StringEscapeUtils.escapeJava(String.valueOf(propertyEntry.getValue())) + "\");");
			}

			block.directStatement("// Special Properties");
			Repository repo = new RepositoryBuilder().findGitDir(mavenProject.getBasedir()).build();
			String branch = repo.getFullBranch();

			Git git = new Git(repo);

			block.directStatement("put(\"maven.project.groupId\", \"" + StringEscapeUtils.escapeJava(mavenProject.getGroupId()) + "\");");
			block.directStatement("put(\"maven.project.artifactId\", \"" + StringEscapeUtils.escapeJava(mavenProject.getArtifactId()) + "\");");
			block.directStatement("put(\"maven.project.version\", \"" + StringEscapeUtils.escapeJava(mavenProject.getVersion()) + "\");");
			block.directStatement("put(\"git.branch\", \"" + StringEscapeUtils.escapeJava(branch) + "\");");
			block.directStatement("put(\"git.branch.clean\", \"" + git.status().call().isClean() + "\");");
			block.directStatement("put(\"git.branch.version\", \"" + StringEscapeUtils.escapeJava(repo.resolve(repo.getFullBranch()).getName()) + "\");");

			directory = "target/generated-sources/module-signer-source/";

			mavenProject.addCompileSourceRoot(directory);

			// output services file
			FileUtils.write(new File(rsrcDirectory, IMavenProject.class.getCanonicalName()), dc.fullName());

			srcDirectory.mkdirs();
			cm.build(srcDirectory);
		}
		catch(IOException exc)
		{
			throw new MojoExecutionException("Error saving source file", exc);
		}
		catch (JClassAlreadyExistsException e)
		{
			throw new MojoExecutionException("Error saving source file", e);
		}
		catch (GitAPIException e)
		{
			throw new MojoExecutionException("Error reading GIT properties", e);
		}
	}

	private String clean(String version)
	{
		return version.replace('.', '_').replace('-', '_');
	}
}